/*
 *  matiec - a compiler for the programming languages defined in IEC 61131-3
 *
 *  Copyright (C) 2003-2011  Mario de Sousa (msousa@fe.up.pt)
 *  Copyright (C) 2007-2011  Laurent Bessard and Edouard Tisserant
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * This code is made available on the understanding that it will not be
 * used in safety-critical situations without a full and competent review.
 *
 *
 * Minimal standaloen C runtime, for test purpose.
 * TODO : UPDATE
 *
 */


#include <stdio.h>
#include <string.h>
#include <time.h>
#include<sys/time.h>
#include <signal.h>

#include "iec_types.h"
#include "iec_std_lib.h"



/*
 * Functions and variables provied by generated C softPLC
 **/ 
extern int common_ticktime__;

IEC_BOOL __DEBUG;

/*
 * Functions and variables provied by plc.c
 **/ 
void config_init__(void);
void run(long int tv_sec, long int tv_nsec);

#define maxval(a,b) ((a>b)?a:b)


void timer_notify(sigval_t val)
{
    struct timespec CURRENT_TIME;
    clock_gettime(CLOCK_REALTIME, &CURRENT_TIME);
    run(CURRENT_TIME.tv_sec, CURRENT_TIME.tv_nsec);
}

void catch_signal(int sig)
{
  signal(SIGTERM, catch_signal);
  signal(SIGINT, catch_signal);
  printf("Got Signal %d\n",sig);
}

int main(int argc,char **argv)
{
    timer_t timer;
    struct sigevent sigev;
    //long tv_nsec = 1000000 * (maxval(common_ticktime__,1)%1000);
    //time_t tv_sec = common_ticktime__/1000;
    long tv_nsec = common_ticktime__ % 1000000000;
    time_t tv_sec =  common_ticktime__ / 1000000000;
 
    struct itimerspec timerValues;
    
    memset (&sigev, 0, sizeof (struct sigevent));
    memset (&timerValues, 0, sizeof (struct itimerspec));
    sigev.sigev_value.sival_int = 0;
    sigev.sigev_notify = SIGEV_THREAD;
    sigev.sigev_notify_attributes = NULL;
    sigev.sigev_notify_function = timer_notify;
    timerValues.it_value.tv_sec = tv_sec;
    timerValues.it_value.tv_nsec = tv_nsec;
    timerValues.it_interval.tv_sec = tv_sec;
    timerValues.it_interval.tv_nsec = tv_nsec;

    printf("common_ticktime__: %ld ns\n", common_ticktime__);

    printf("\n - timerValues - \n");
    printf("interval tv_sec = %u, tv_nsec = %u\n", timerValues.it_interval.tv_sec, timerValues.it_interval.tv_nsec); 
    printf("config_init__();\n");
    config_init__();

    int err = timer_create (CLOCK_REALTIME, &sigev, &timer);

    if (err < 0) {
        printf("Error while creating timer\n");
    }
    timer_settime (timer, 0, &timerValues, NULL);

    
#if 0
    struct timespec CURRENT_TIME;
    clock_gettime(CLOCK_REALTIME, &CURRENT_TIME);
    run(CURRENT_TIME.tv_sec, CURRENT_TIME.tv_nsec);
 #endif

    
    /* install signal handler for manual break */
    signal(SIGTERM, catch_signal);
    signal(SIGINT, catch_signal);
    
    pause();
    
    timer_delete (timer);
    
    return 0;
}

